#!/usr/bin/perl
#
# Name: logger.pl
# Copyright: (c)Copyright 2008 Hewlett-Packard Development Company, L.P.
#
# Description: Logs messages into /var/cpq/Component.log
#
# Modification History
#
# Chad Dupuis	02/29/08 Initial development
# Chad Dupuis	03/24/08 Added CreateLogFile sub to check and create the
#                        log file if it doesn't exist

#
# Defines
#

# If the environmental variable HPFCHBAKITDEBUG is "y" then
# print debug statements

if ($ENV{HPFCHBAKITDEBUG} eq "y") {
	$debug = 1;
}
else {
	$debug = 0;
}

# Actions that script can do

$action_none = 0; # No action specified
$action_start_session = 1; # Write session header
$action_stop_session = 2; # Write session footer
$action_append_message = 3; # Write message to log file

# Name of log file
$logfile = "/var/log/hp-fc-enablement.log";

# Message to send to logfile
$logmessage = "";

#
# Functions
#

# Name: Debug
# Description: Prints a debug statement
# In: function - Name of the function that this statement is generated from
#     message - Message to print
# Out: None
# Returns: None
sub Debug {
	my $function = $_[0];
	my $message = $_[1];

	if ($debug) {	
		print $function."(): ".$message."\n";
	}
}

# Name: PrintHelp
# Description: Prints the help message
# In: None
# Out: None
# Returns: None
sub PrintHelp {
	print "NAME\n\n";
	print "logger.pl\n\n";
	print "DESCRIPTION\n\n";
	print "Logging facility for hp-fc-enablement RPM\n\n";
	print "OPTIONS\n\n";
	print "-s, --start   - Start a new log session\n";
	print "-e, --stop    - End a new log session\n";
	print "-m, --message - Print a new message to the log\n";
	exit ($GOOD);
}

# Name: ParseArgs
# Description: Parse all command line arguments and return the appropriate
#              action to take
# In: @paramters - command line arguments
# Out: None
# Returns: Action to take
sub ParseArgs {
	@parameters = @_;
	my $size = @parameters;
	my $i = 0;
	my $action = $action_none;
	
	Debug ("ParseArgs", "Number of parameters is $size");
	
	for ($i = 0; $i < $size; $i = $i + 1)  
	{
		my $arg = $parameters[$i];
		Debug ("ParseArgs", "Current paramter is $arg");
		
		if (($arg eq "-h") || ($arg eq "--help")) {
			PrintHelp();
		}
		elsif (($arg eq "-d") || ($arg eq "--debug")) {
			$debug = 1; # Turn debug on
		}
		elsif (($arg eq "-s") || ($arg eq "--start")) {
			$action = $action_start_session;
		}
		elsif (($arg eq "-e") || ($arg eq "--stop")) {
			$action = $action_stop_session;
		}
		elsif (($arg eq "-m") || ($arg eq "--message")) {
			$action = $action_append_message;
			$i++;
			$logmessage = $parameters[$i];
			Debug ("ParseArgs", "logmessage is $logmessage")
		}
		else {
			print "$arg is an invalid flag\n";
			exit ($FAIL);
		}
	}
	
	# Return the action for the script to take

	Debug ("ParseArgs", "Returning $action");	
	return ($action);
}

# Name: StartSession
# Description: Prints the session header to $logfile
# In: None
# Out: None
# Returns: None
sub StartSession {
	
	# Open for append
	
	Debug ("StartSession", "logfile is $logfile");
	open (LOGFILE, ">>$logfile") or die "Could not open $logfile\n";
	
	# Write header
	
	my $curtime = `date`;
	Debug ("StartSession", "Current time is $curtime");
	Debug ("StartSession", "Writing \"HP Fibre Channel Enablement Kit Installation Session\n\" to $logfile");
	print LOGFILE "HP Fibre Channel Enablement Kit Installation Session\n";
	Debug ("StartSession", "Writing \"====================================================\n\n\" to $logfile");
	print LOGFILE "====================================================\n\n";
	Debug ("StartSession", "Writing \"Session start: $curtime\n\" to $logfile");
	print LOGFILE "Session start: $curtime\n";
	
	# Close the file
	close (LOGFILE);	
}

# Name: StopSession
# Description: Prints the session footer to $logfile
# In: None
# Out: None
# Returns: None
sub StopSession {
	
	# Open for append
	
	Debug ("StopSession", "logfile is $logfile");
	open (LOGFILE, ">>$logfile") or die "Could not open $logfile\n";
	
	# Write header
	
	my $curtime = `date`;
	Debug ("StopSession", "Current time is $curtime");
	Debug ("StopSession", "Writing line \"\nSession end:   $curtime\n\" to $logfile");
	print LOGFILE "\nSession end:   $curtime\n";
	
	# Close the file
	close (LOGFILE);	
}

# Name: WriteLogMessage
# Description: Prints the text in $logmessage to $logfile
# In: None
# Out: None
# Returns: None
sub WriteLogMessage {
	# Check to make sure that we have a message to print
	
	if ($logmessage eq "") {
		Debug ("WriteLogMessage", "No logmessage, exiting subroutine");
		return;
	}
	
	# Open for append
	
	Debug ("WriteLogMessage", "logfile is $logfile");
	open (LOGFILE, ">>$logfile") or die "Could not open $logfile\n";
	
	# Write message
	
	Debug ("WriteLogMessage", "Writing \"$logmessage\n\"");
	print LOGFILE "$logmessage\n";
	
	# Close the file
	close (LOGFILE);	
}

# Name: CreateLogFile
# Description: Checks for the existance of logfile and if it doesn't exist
#              then it is created
# In: None
# Out: None
# Returns: None
sub CreateLogFile {
	if (!(-e $logfile)) {
		Debug ("CreateLogFile", "logfile is $logfile");
		open (LOGFILE, ">$logfile");
		close (LOGFILE);
	}
}

#
# Main
#

# Save command line arguments
@arguments = @ARGV;

# Create the log file
CreateLogFile();

# Determine the action to take

$action_to_take = ParseArgs(@arguments);
Debug ("main", "action_to_take is $action_to_take");

if ($action_to_take == $action_start_session) {
	StartSession();
}
elsif ($action_to_take == $action_stop_session) {
	StopSession();
}
elsif ($action_to_take == $action_append_message) {
	WriteLogMessage();
}

Debug ("main", "Returning GOOD");
exit ($GOOD);
